@@ -23,6 +23,7 @@ import("core.base.option")
 import("core.base.tty")
 import("core.package.package", {alias = "core_package"})
 import("core.project.target")
+import("core.platform.platform")
 import("lib.detect.find_file")
 import("private.action.require.impl.actions.test")
 import("private.action.require.impl.actions.patch_sources")
@@ -74,7 +75,7 @@ function _patch_pkgconfig(package)
 
     -- cflags
     local cflags = ""
-    for _, includedir in ipairs(fetchinfo.includedirs) do
+    for _, includedir in ipairs(fetchinfo.includedirs or fetchinfo.sysincludedirs) do
         if includedir ~= path.join(installdir, "include") then
             cflags = cflags .. " -I" .. (includedir:gsub("\\", "/"))
         end
@@ -222,16 +223,6 @@ function _fix_paths_for_precompiled_package(package)
     end
 end
 
-
--- check package toolchains
-function _check_package_toolchains(package)
-    for _, toolchain_inst in pairs(package:toolchains()) do
-        if not toolchain_inst:check() then
-            raise("toolchain(\"%s\"): not found!", toolchain_inst:name())
-        end
-    end
-end
-
 -- get failed install directory
 function _get_installdir_failed(package)
     return path.join(package:cachedir(), "installdir.failed")
@@ -316,6 +307,43 @@ function _leave_workdir(package, oldir)
     _clear_sourcedir(package)
 end
 
+-- enter package install environments
+function _enter_package_installenvs(package)
+    for _, dep in ipairs(package:orderdeps()) do
+        dep:envs_enter()
+    end
+end
+
+-- enter package test environments
+function _enter_package_testenvs(package)
+
+    -- add compiler runtime library directory to $PATH
+    -- @see https://github.com/xmake-io/xmake-repo/pull/3606
+    if is_host("windows") and package:is_plat("windows", "mingw") then -- bin/*.dll for windows
+        local toolchains = package:toolchains()
+        if not toolchains then
+            local platform_inst = platform.load(package:plat(), package:arch())
+            toolchains = platform_inst:toolchains()
+            for _, toolchain_inst in ipairs(toolchains) do
+                if toolchain_inst:check() then
+                    local runenvs = toolchain_inst:runenvs()
+                    if runenvs and runenvs.PATH then
+                        local envs = {PATH = runenvs.PATH}
+                        os.addenvs(envs)
+                    end
+                end
+            end
+        end
+    end
+
+    -- enter package environments
+    for _, dep in ipairs(package:orderdeps()) do
+        dep:envs_enter()
+    end
+    package:envs_enter()
+end
+
+
 function main(package)
 
     -- enter working directory
@@ -344,7 +372,7 @@ function main(package)
             else
 
                 -- build and install package to the install directory
-                local force_reinstall = package:data("force_reinstall") or option.get("force")
+                local force_reinstall = package:policy("package.install_always") or package:data("force_reinstall") or option.get("force")
                 if force_reinstall or not package:manifest_load() then
 
                     -- clear install directory
@@ -357,12 +385,7 @@ function main(package)
                     patch_sources(package)
 
                     -- enter the environments of all package dependencies
-                    for _, dep in ipairs(package:orderdeps()) do
-                        dep:envs_enter()
-                    end
-
-                    -- check package toolchains
-                    _check_package_toolchains(package)
+                    _enter_package_installenvs(package)
 
                     -- do install
                     if script ~= nil then
@@ -385,10 +408,7 @@ function main(package)
             end
 
             -- enter the package environments
-            for _, dep in ipairs(package:orderdeps()) do
-                dep:envs_enter()
-            end
-            package:envs_enter()
+            _enter_package_testenvs(package)
 
             -- fetch package and force to flush the cache
             local fetchinfo = package:fetch({force = true})
